/*
 * @(#)file      Agent.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.10
 * @(#)lastedit  04/04/19
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// java import
//
import java.net.InetAddress;
import java.util.Vector;

// RI import
//
import javax.management.ObjectName;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpString;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.manager.SnmpPeer;
import com.sun.management.snmp.manager.SnmpParameters;

// jdmk import
//
import com.sun.management.snmp.SnmpOidTableSupport;
import com.sun.jdmk.comm.HtmlAdaptorServer;
import com.sun.management.comm.SnmpAdaptorServer;
import com.sun.management.comm.SnmpInformRequest;
import com.sun.management.comm.SnmpInformHandler;

/**
 * The Agent class provides a simple example on how to use the SNMP 
 * adaptor server in order to send an inform request. In this example,
 * the inform request is used as an "acknowledged trap".
 *
 * <p>When calling the program, you must specify:
 * <br>  - host: hostname of the SNMP manager you want to send the inform to.
 *
 * <p>The port number in use for sending inform PDUs is 8085.
 */

public class Agent {

    /**
     * Main entry point.
     * When calling the program, you must specify the
     * hostname of the SNMP manager you want to send the inform to.
     */
    public static void main(String argv[]) {
        
        final MBeanServer server;
        final ObjectName htmlObjName;
        final ObjectName snmpObjName;
        final int htmlPort = 8082;
        final int snmpPort = 8083;
        
        if (argv.length != 2) {
            usage();
            java.lang.System.exit(1);
        }
    
        final String host = argv[0];
                
        // Initialize the port number to send inform PDUs.
        //
        final int port = Integer.parseInt(argv[1]);

        try {
            server = MBeanServerFactory.createMBeanServer();
            String domain = server.getDefaultDomain();

            // Create and start the HTML adaptor.
            //
            htmlObjName = new ObjectName(domain + 
                 ":class=HtmlAdaptorServer,protocol=html,port=" + htmlPort);
            HtmlAdaptorServer htmlAdaptor = new HtmlAdaptorServer(htmlPort);
            server.registerMBean(htmlAdaptor, htmlObjName);
            htmlAdaptor.start();
                        
            // Create and start the SNMP adaptor.
            //
            snmpObjName = new ObjectName(domain + 
                 ":class=SnmpAdaptorServer,protocol=snmp,port=" + snmpPort);
            SnmpAdaptorServer snmpAdaptor = new SnmpAdaptorServer(snmpPort);
            server.registerMBean(snmpAdaptor, snmpObjName);
            snmpAdaptor.start();
                  
            snmpAdaptor.setInformPort(port);
            println("\nNOTE: SNMP adaptor server initialized");
            
            // We do not need to instantiate the MIB into the MBean server.
            // We just use the SNMP oid table to construct the varbind 
	    // list of the inform request.
            //
            final SnmpOidTableSupport oidTable = new RFC1213_MIBOidTable();
            SnmpOid.setSnmpOidTable(oidTable);
            final SnmpVarBindList list = 
		new SnmpVarBindList("Agent varbind list");
            final SnmpVarBind avar1 = 
		new SnmpVarBind(new SnmpOid("sysDescr.0"), 
				new SnmpString("! My sysDescr !")) ;
            list.addVarBind(avar1);
           
            
            // Create a SnmpPeer 
	    // Use the address and port. 
	    //Doing so, you can send inform to manager listening to different
	    // ports.
	    SnmpPeer peer = new SnmpPeer(host, port);
	    
	    //Create the peer parameters.
	    SnmpParameters p = new SnmpParameters();

	    // Set the inform community string.
	    p.setInformCommunity("public");

	    // Set the parameters in the peer.
	    peer.setParams(p);
	    
            // Create a simple implementation of an SnmpInformHandler. 
            //
            final AgentRspHandler handler = new AgentRspHandler();
            
            // Make the SNMP inform request in asynchronous mode.
            //
            println("\n>> Press <ENTER> to send the inform request on " + 
		    host + " at port " + port + "...");
            java.lang.System.in.read();
            SnmpInformRequest request = 
		snmpAdaptor.snmpInformRequest(peer,
					      handler, 
					      new SnmpOid("1.2.3.4"), 
					      list);
            println("NOTE: Inform request sent to SNMP manager on " + 
		    host + " at port " + port);
                        
            // Here you could do whatever processing you need, for 
	    // instance you could run a graphical user interface.
            //
            Thread.sleep(2000);
            
            println("\n>> Press <Enter> to stop this SNMP adaptor server.");
            java.lang.System.in.read();
            snmpAdaptor.stop();
            
            println("NOTE: SNMP adaptor server stopped...");
            
            java.lang.System.exit(0);            
        
        } catch (Exception e) {
            java.lang.System.err.println("Agent::main: Exception occurred:"+
					 e );
            e.printStackTrace();
        }
    }

    /**
     * Return usage of the program.
     */
    public static void  usage() {
        println("java Agent <hostname> <port>");
        println("where");
        println("\t-hostname: hostname or IP address of the SNMP manager" +
		" you want to send the inform to.");
	println("\t-port: port of the SNMP manager" +
		" you want to send the inform to.");
    }

    /**
     * print/println stuff...
     **/
    private final static void println(String msg) {
	java.lang.System.out.println(msg);
    }
    private final static void print(String msg) {
	java.lang.System.out.print(msg);
    }

}
