/*
 * @(#)file      AgentV3.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.8
 * @(#)lastedit  04/04/19
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// java import
//
import java.net.InetAddress;
import java.util.Vector;

// RI import
//
import javax.management.ObjectName;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpString;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.SnmpDefinitions;

// Manager API part needed by SNMP V3 inform sending.
import com.sun.management.snmp.manager.usm.SnmpUsmParameters;
import com.sun.management.snmp.manager.usm.SnmpUsmPeer;

// jdmk import
//
import com.sun.management.snmp.SnmpOidTableSupport;
import com.sun.jdmk.comm.HtmlAdaptorServer;

// V3 adaptor and request.
import com.sun.management.comm.SnmpV3AdaptorServer;
import com.sun.management.comm.SnmpV3InformRequest;

import com.sun.management.comm.SnmpInformHandler;

/**
 * The AgentV3 class provides a simple example on how to use the 
 * SNMP V3 adaptor server to send an inform request.
 *
 * <p>When calling the program, you must specify:
 * <br> - host: hostname of the SNMP manager you want to send the inform to.
 *
 * <p>The port number in use for sending inform PDUs is 8085.
 */

public class AgentV3 {

    /**
     * Main entry point.
     * When calling the program, you must specify the
     * hostname of the SNMP manager you want to send the inform to.
     */
    public static void main(String argv[]) {
        
        final MBeanServer server;
        final ObjectName htmlObjName;
        final ObjectName snmpObjName;
        final int htmlPort = 8082;
        final int snmpPort = 8083;
        
        
        if (argv.length != 2) {
            usage();
            java.lang.System.exit(1);
        }
    
        final String host = argv[0];
        
	// Initialize the port number to send inform PDUs.
        //
        final int port = Integer.parseInt(argv[1]);

        try {
            server = MBeanServerFactory.createMBeanServer();
            final String domain = server.getDefaultDomain();

            // Create and start the HTML adaptor.
            //
            htmlObjName = new ObjectName(domain + 
                 ":class=HtmlAdaptorServer,protocol=html,port=" + htmlPort);
            final HtmlAdaptorServer htmlAdaptor = 
		new HtmlAdaptorServer(htmlPort);
            server.registerMBean(htmlAdaptor, htmlObjName);
            htmlAdaptor.start();
                        
            // Create and start the SNMP adaptor.
            //
            snmpObjName = new ObjectName(domain + 
                  ":class=SnmpAdaptorServer,protocol=snmp,port=" + snmpPort);
            final SnmpV3AdaptorServer snmpAdaptor = 
		new SnmpV3AdaptorServer(snmpPort);
	    snmpAdaptor.setInformPort(port);
            server.registerMBean(snmpAdaptor, snmpObjName);
            snmpAdaptor.start();
            
            println("\nNOTE: SNMP adaptor server initialized");
            
            // We do not need to instantiate the MIB into the MBean server.
            // We just use the SNMP oid table to construct the varbind 
	    // list of the inform request.
            //
            final SnmpOidTableSupport oidTable = new RFC1213_MIBOidTable();
            SnmpOid.setSnmpOidTable(oidTable);
            final SnmpVarBindList list = 
		new SnmpVarBindList("Agent varbind list");
            final SnmpVarBind avar1 = 
		new SnmpVarBind(new SnmpOid("sysDescr.0"), 
				new SnmpString("! My sysDescr !")) ;
            list.addVarBind(avar1);
            
            // Retrieve the destination host.
            //
            final InetAddress address = InetAddress.getByName(host);
            

	    // Create a USM peer that will be used when sending the inform
	    // Passing it the manager address and inform port.
	    // The inform port is the same as the one provided to the 
	    // SnmpV3AdaptorServer via setInformPort().
	    //
	    final SnmpUsmPeer peer = new SnmpUsmPeer(snmpAdaptor.getEngine(),
						     address.getHostName(), 
						     port);
            
	    // Create the security parameters that are used when sending 
	    // the inform.
	    //
	    SnmpUsmParameters params = 
		new SnmpUsmParameters(snmpAdaptor.getEngine());
	    
	    // Set the parameters
	    //
	    
	    // First a principal
	    params.setPrincipal("defaultUser");
	    
	    // A security level. Authentication is applied.
	    params.setSecurityLevel(SnmpDefinitions.authNoPriv);
	    
	    // Add a contextEngineId. The context engine Id is the 
	    // manager engine Id one and not the adaptor one.
	    // This is tricky but when you send inform, the authoritative 
	    // engine is the receiver (manager in our case).
	    //
	    params.setContextEngineId(peer.getEngineId().getBytes());

	    //Add a context name.
	    //
	    params.setContextName("TEST-CONTEXT".getBytes());
	    
	    // Set the parameters in the peer.
	    //
	    peer.setParams(params);
	    

	    // The inform is an authenticated one, the timeliness 
	    // discovery must be processed.
	    //
	    peer.processUsmTimelinessDiscovery();

            // Create a simple implementation of an SnmpInformHandler. 
            //
            AgentRspHandler handler = new AgentRspHandler();
            
            // Make the SNMP inform request in asynchronous mode.
            //
            println("\n>> Press <ENTER> to send the inform request on " + 
		    host + " at port " + port + "...");
            java.lang.System.in.read();
            SnmpV3InformRequest request = 
		snmpAdaptor.snmpV3UsmInformRequest(peer, handler, 
					     new SnmpOid("1.2.3.4"), list);
            println("NOTE: Inform request sent to SNMP manager on " + host +
		    " at port " + port);
                        
            // Here you could do whatever processing you need, for 
	    // instance you could run a graphical user interface.
            //
            Thread.sleep(2000);
            
            println("\n>> Press <Enter> to stop this SNMP adaptor server.");
            java.lang.System.in.read();
            snmpAdaptor.stop();
            
            println("NOTE: SNMP adaptor server stopped...");
            
            java.lang.System.exit(0);            
        
        } catch (Exception e) {
            java.lang.System.err.println("AgentV3::main: " +
					 "Exception occurred:" + e );
            e.printStackTrace();
        }
    }

    /**
     * Return usage of the program.
     */
    public static void  usage() {
        println("java AgentV3 <hostname>");
        println("where");
        println("\t-hostname: hostname or IP address of the SNMP manager" + 
		" you want to send the inform to.");
        println("The port number in use is 8085.");
    }
    
    /**
     * print/println stuff...
     **/
    private final static void println(String msg) {
	java.lang.System.out.println(msg);
    }
    private final static void print(String msg) {
	java.lang.System.out.print(msg);
    }

}
