/*
 * @(#)file      SimpleManager1V3.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.6
 * @(#)lastedit  04/04/07
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// java import
//
import java.util.Vector;
import java.util.Enumeration;
import java.io.IOException;

// RI import
//
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.manager.usm.SnmpUsmPeer;
import com.sun.management.snmp.manager.usm.SnmpUsmParameters;
import com.sun.management.snmp.manager.SnmpRequest;
import com.sun.management.snmp.manager.SnmpSession;

/**
 * The class is a simple example showing how to use the SNMP V3 Manager API
 * and particularly how to send an inform request.
 *
 * <p>When calling the program, you must specify:
 * <br> - host: hostname of the SNMP manager you want to send the inform to.
 *
 * <p>The port number in use for sending inform PDUs is 8085.
 */

public class SimpleManager1V3 {
   
    /**
     * Main entry point.
     * <p>When calling the program, you must specify the
     * hostname of the SNMP manager you want to send the inform to.
     */
    public static void main(String argv[]) {
     
        if (argv.length != 1) {
            usage();
            java.lang.System.exit(1);
        }
    
        final String host = argv[0];
        
        // Initialize the port number to send inform PDUs on port 8085.
        //
        final int port = 8085;

        try {
	    // Build the session. A session creates, controls and manages 
	    // one or more requests.
            //
            final SnmpSession session= 
		new SnmpSession("SimpleManager1V3 session");

            // Create an SnmpPeer object for representing the entity 
	    // to communicate with. 
            //
            final SnmpUsmPeer peer = new SnmpUsmPeer(session.getEngine(),
						     host, 
						     port);
     
            // Create parameters to associate to the entity to 
	    // communicate with.
            // When creating the parameter object, you specify the needed 
	    // security related parameters.
            // 
            final SnmpUsmParameters params = 
		new SnmpUsmParameters(session.getEngine());

	    // Set the parameters
	    //
	    
	    // First a principal
	    //
	    params.setPrincipal("defaultUser");
	    
	    // A security level. Authentication is applied.
	    // 
	    params.setSecurityLevel(SnmpDefinitions.authNoPriv);
	    
	    // Add a contextEngineId. The context engine Id is the 
	    // manager engine Id one and not the adaptor one.
	    // This is tricky but when you send inform, the authoritative
	    // engine is the receiver (manager in our case).
	    //
	    params.setContextEngineId(peer.getEngineId().getBytes());

	    // Add a context name.
	    //
	    params.setContextName("TEST-CONTEXT".getBytes());
	    
            // The newly created parameter must be associated to the peer.
            //
            peer.setParams(params);
     

	    // The inform is an authenticated one, the timeliness 
	    // discovery must be processed.
	    //
	    peer.processUsmTimelinessDiscovery();
	    


            // A default peer (listener manager) can be associated to an 
	    // SnmpSession. When invoking a service provided by the 
	    // SnmpSession, if the peer is not specified, the session 
	    // will perform the service using the default one as the 
	    // target of the service
            //
            // NOTE:
            // Take care if you want to use the same SnmpSession object 
	    // to communicate with both agent  (for get/set/getNext 
	    // requests) and manager (for inform requests).
            // Indeed, you can specify only 1 default peer which 
	    // corresponds either to the agent or to the manager !
            //
            session.setDefaultPeer(peer);
	    
            println("\nNOTE: SNMP V3 simple manager 1 initialized");
            
            // Make the SNMP inform request and wait for the result.
            //
            println("\n>> Press <ENTER> to send the inform request on " + 
		    host + " at port " + port + "...");
            try {
                System.in.read();
            } catch (IOException e) {
                e.printStackTrace();
            }

            final SnmpRequest request = 
		session.snmpInformRequest(null, new SnmpOid("1.2.3.4"), 
					  null);

            println("NOTE: Inform request sent to SNMP manager on " + 
		    host + " at port " + port);

            final boolean completed = request.waitForCompletion(10000);
       
            // Check for a timeout of the request.
            //
            if (completed == false) {
                println("\nSimpleManager1::main: Request timed out." +
			" Check reachability of agent");
	 
                // Print request.
                //
                println("Request: " + request.toString());
                java.lang.System.exit(0);
            }
     
            // Now we have a response. Check if the response contains 
	    // an error.
            //
            final int errorStatus = request.getErrorStatus();
            if (errorStatus != SnmpDefinitions.snmpRspNoError) {
                println("Error status = " +
			SnmpRequest.snmpErrorToString(errorStatus));
                println("Error index = " + request.getErrorIndex());
                java.lang.System.exit(0);
            }
       
            // Now we shall display the content of the result.
            //
            final SnmpVarBindList result = request.getResponseVarBindList();
            println("\nNOTE: Response received:\n" + result);
       
            // Nicely stop the session
            //
            println("\nNOTE: SNMP V3 simple manager 1 stopped...");
            session.destroySession();
       
            //
            // That's all !
            //
            java.lang.System.exit(0);
     
        } catch(Exception e) {
            java.lang.System.err.println("SimpleManager1V3::main: " +
					 "Exception occurred:" + e );
            e.printStackTrace();
        }
    }
   
    /**
     * Return usage of the program.
     */
    public static void  usage() {
        println("java SimpleManager1V3 <hostname>");
        println("where");
        println("\t-hostname: hostname or IP address of the " +
		"SNMP V3 manager you want to send the inform to.");
        println("The port number in use is 8085.");
    }

    /**
     * print/println stuff...
     **/
    private final static void println(String msg) {
	java.lang.System.out.println(msg);
    }

    private final static void print(String msg) {
	java.lang.System.out.print(msg);
    }

}
