/*
 * @(#)file      JmxMBeanAttrTableMetaImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.5
 * @(#)lastedit  04/04/07
 * 
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// java imports
//
import java.io.Serializable;
import java.util.Vector;

// jmx imports
//
import javax.management.MBeanServer;
import javax.management.ObjectName;


// jdmk imports
//
import com.sun.management.snmp.SnmpCounter;
import com.sun.management.snmp.SnmpCounter64;
import com.sun.management.snmp.SnmpGauge;
import com.sun.management.snmp.SnmpInt;
import com.sun.management.snmp.SnmpUnsignedInt;
import com.sun.management.snmp.SnmpIpAddress;
import com.sun.management.snmp.SnmpTimeticks;
import com.sun.management.snmp.SnmpOpaque;
import com.sun.management.snmp.SnmpString;
import com.sun.management.snmp.SnmpStringFixed;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpNull;
import com.sun.management.snmp.SnmpValue;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpStatusException;

import com.sun.management.snmp.agent.SnmpIndex;
import com.sun.management.snmp.agent.SnmpMib;
import com.sun.management.snmp.agent.SnmpMibTable;
import com.sun.management.snmp.agent.SnmpMibSubRequest;
import com.sun.management.snmp.agent.SnmpStandardObjectServer;

/**
 * The class is used for implementing the "JmxMBeanAttrTable" meta.
 * You must understand the "JmxMBeanTableMetaImpl" class before looking
 * at this.
 *
 * The JmxMBeanAttrTableMetaImpl class extends the generated 
 * JmxMBeanAttrTableMeta class in order to override the following methods:
 *
 *  - getEntry(SnmpOid rowOid): returns an JmxMBeanAttrEntryImpl object
 *    for the indicated row.
 *  - contains(SnmpOid oid, Object userData): Tells whether the indicated
 *    row is present int the table.
 *  - getNextOid(Object userData): returns the rowOid (row index) of the
 *    first row in the table.
 *  - getNextOid(SnmpOid rowOid,Object userData): returns the index of 
 *    the row that immediately follows the indicated row in the table.
 *
 * This class does not use any specific caching mechanism. Since the
 * jmxMBeanAttrTable is an extension of the jmxMBeanTable it simply relies
 * on the cache established for the jmxMBeanTable. The list of attributes
 * pertaining to a specific MBean is cached (if needed) in the corresponding
 * JmxMBeanEntryImpl object, which will be reclaimed at the end of the
 * request. Note that attribute values are not cached - they are obtained
 * when needed.
 *
 **/
public class JmxMBeanAttrTableMetaImpl extends JmxMBeanAttrTableMeta {

    /** 
     * The meta of the jmxMBeanTable
     **/
    private JmxMBeanTableMetaImpl jmxMBeanTableMeta = null;


    /** 
     * The MIB object
     **/
    private JMX_MBEAN_SERVER_MIB_Impl myMib = null;

    /**
     * Constructor for the table. Initialize metadata for 
     * "JmxMBeanAttrTableMetaImpl".
     */
    public JmxMBeanAttrTableMetaImpl(SnmpMib myMib, 
				     SnmpStandardObjectServer objserv) {
        super(myMib,objserv);
	this.myMib = (JMX_MBEAN_SERVER_MIB_Impl)myMib;
    }

    /**
     * Returns the meta of the jmxMBeanTable. Makes a lazy evaluation of
     * the jmxMBeanTableMeta variable. This is necessary because 
     * myMib.getJmxMBeanTableMeta() is not guaranteed to work until
     * myMib.init() has successfully completed. 
     * Since the meta objects are constructed during that init() phase,
     * calling myMib.getJmxMBeanTableMeta() in this object constructor
     * would not be safe (might return null).
     **/
    public synchronized JmxMBeanTableMetaImpl getJmxMBeanTableMeta() {
	if (jmxMBeanTableMeta == null) {
	    jmxMBeanTableMeta = myMib.getJmxMBeanTableMeta();
	}
	return jmxMBeanTableMeta;
    }


    // Returns the rowOid (row index) of the first row in the table.
    // See com.sun.jmx.snmp.agent.SnmpMibTable
    //
    protected SnmpOid getNextOid(Object userData) 
	throws SnmpStatusException {
	// null means get the first OID.
	return getNextOid(null,userData);
    }

    // Returns the index of the row that immediately follows the
    // indicated row in the table.
    // See com.sun.jmx.snmp.agent.SnmpMibTable
    //
    protected SnmpOid getNextOid(SnmpOid oid, Object userData)
	throws SnmpStatusException {
	final boolean dbg = Boolean.getBoolean("debug");
	if (dbg) System.err.println("getNextOid: previous attr=" + oid);

	// Get the cached data for the jmxMBeanTable. We rely on the 
	// caching mechanism implemented for the jmxMBeanTable.
	//
	JmxMBeanTableMetaImpl.JmxMBeanTableData tableData = 
	    getJmxMBeanTableMeta().getTableDatas(userData);

	if (tableData == null) {
	    // This should never happen.
	    // If we get here it's a bug.
	    //
	    if (dbg) System.err.println("getNextOid: tableData is null!");
	    throw new SnmpStatusException(SnmpStatusException.noSuchInstance);
	}
	
	// Now we're going to attempt to find out the jmxMBeanEntryImpl object
	// to which the next jmxMBeanAttrEntryImpl we're looking for belong.
	//

	// Initialize the next oid
	//
	SnmpOid next = oid;
	

	// Loop until we manage to pre-load the MBean data of the next.
	// 
	while (true) {

	    // Determine the index of the MBean to look at:
	    //   We know that a complete rowOid is of the form:
	    //   <MBeanIndex>.<AttributeIndex>
	    //
	    final SnmpOid mbeanOid;
	    if (next != null && next.getLength() > 0) {
		// The MBeanIndex is contained in the input OID: 
		// extract the MBeanIndex
		//
		mbeanOid = new SnmpOid(next.getOidArc(0));
	    } else {
		// No MBeanIndex => take first (= lowest index)
		//
		mbeanOid = tableData.getNext(null);
	    }

	    if (dbg) 
		System.err.println("getNextOid: mbean index is: " + mbeanOid);

	    if (mbeanOid == null) 
		throw new 
		    SnmpStatusException(SnmpStatusException.noSuchInstance);

	    // We have know the potential MBean. The next attribute we
	    // are looking for may be in that MBean (if the last attribute
	    // was not yet reached), or in the next MBean (if the last 
	    // attribute was already reached).
	    //
	    // Determine the index of the next attribute
	    //
	    final int attrIndex;

	    if (next == null ||  next.getLength() < 2) {
		// If the attribute index it is not contained in the input OID,
		// the next attribute is the first one.
		//
		attrIndex = 1;
	    } else {
		// Otherwise, it is the contained index+1
		//
		attrIndex = (int)next.getOidArc(1)+1;
	    }
	    if (dbg) 
		System.err.println("getNextOid: attr index is: " + attrIndex);

	    try {
		// Now load the MBean 
		//
		final JmxMBeanEntryImpl mbeanEntry = (JmxMBeanEntryImpl)
		    getJmxMBeanTableMeta().getEntry(mbeanOid);

		// Then load the attribute
		//
		mbeanEntry.getAttribute(attrIndex);

		// No error? we got it.
		//
		next = new SnmpOid(new long[] {
		    mbeanOid.getOidArc(0),(long)attrIndex});
		break;
	    } catch (SnmpStatusException x) {
		// We got an exception => we have an out of bound index,
		// or we couldn't load the MBean => skip to next MBean
		// (simply let flow to the end of the loop)
		// 
		if (dbg) {
		    System.err.println("getNextOid: failed to get next attr:"+ 
				       " " + mbeanOid + "." + attrIndex );
		}
	    } catch (Exception x) {
		// We got an exception =>  we couldn't load the MBean 
		// or something bad happened => skip to next MBean
		// (simply let flow to the end of the loop)
		// 
		if (dbg) {
		    System.err.println("getNextOid: unexpected exception: "+x);
		    x.printStackTrace();
		}
	    }

	    // The next attribute was not in that MBean. 
	    // Skip to next MBean
	    //
	    if (dbg) 
		System.err.println("getNextOid: skipping to next MBean: "+
				   "previous was: " + mbeanOid);
	    next = tableData.getNext(mbeanOid);
	    if (dbg) 
		System.err.println("getNextOid: next is now: " + next);
	    if (next == null) 
		throw new 
		    SnmpStatusException(SnmpStatusException.noSuchInstance);
	} 
	
	if (dbg) System.err.println("getNextOid: next attr=" + next);
	return next;
    }
   
    // Tells whether the indicated row is present int the table.
    // See com.sun.jmx.snmp.agent.SnmpMibTable
    //
    protected boolean contains(SnmpOid oid, Object userData) {

	try {
	    // Preload the MBean Data in cache.
	    //
	    return (loadInCache(oid) != null);
	} catch (Exception x) {
	    if (Boolean.getBoolean("info")) {
		System.err.println("Failed to load MBean Attribute entry "+
				   "for " + oid + ": "+ x);
	    }
	    if (Boolean.getBoolean("debug")) {
		x.printStackTrace();
	    }
	}

	return false;
    }

    // Returns an JmxMBeanAttrEntryImpl object for the indicated row.
    // See com.sun.jmx.snmp.agent.SnmpMibTable
    //
    public Object getEntry(SnmpOid oid) 
	throws SnmpStatusException {
	return loadInCache(oid);
    }

    // Loads the indicated attribute into the cache (that is, in its 
    // corresponding JmxMBeanEntryImpl). This has the side effect of:
    // - (re)computing the jmxMBeanTable index
    // - uploading that index into the request-contextual cache
    // - creating the JmxMBeanEntryImpl to which this attribute belongs (if 
    //   needed) - the JmxMBeanEntryImpl object is then both stored in the
    //   global cache (JmxMBeanTableMetaImpl.JmxMBeanTableCache) and the
    //   request contextual cache
    // - creating (if needed) and caching the returned JmxMBeanAttrEntryImpl
    //   into its corresponding JmxMBeanEntryImpl
    //   (see JmxMBeanEntryImpl.getAttribute(int)).
    //
    private  JmxMBeanAttrEntry loadInCache(SnmpOid rowOid) 
	throws SnmpStatusException {
	if (rowOid == null || rowOid.getLength() < 2)
	    throw new SnmpStatusException(SnmpStatusException.noSuchInstance);
	
	// We know in the case of this table that the index is built of
	// two integers, it is thus the first OID arc of the index OID.
	//
	final long   mbeanIndex    = rowOid.getOidArc(0);

	// We know in the case of this table that the index is built of
	// two integers, it is thus the second OID arc of the index OID.
	//
	final long   attrIndex    = rowOid.getOidArc(1);

	// Get the MBean Entry. We don't need to get/put anything in the
	// cache because it's taken care of by the JmxMBeanTableMeta.
	// See JmxMBeanTableMetaImpl.getEntry(SnmpOid).
	//
	JmxMBeanEntryImpl mbeanEntry = (JmxMBeanEntryImpl)
	    getJmxMBeanTableMeta().getEntry(new SnmpOid(mbeanIndex));
	

	// mbeanEntry should never be null.
	//
	if (mbeanEntry == null) 
	    throw new SnmpStatusException(SnmpStatusException.noSuchInstance);

	// Get the attribute entry associated with our mbeanEntry at index
	// attrIndex.
	//
	JmxMBeanAttrEntry entry = null;
	try {
	    entry = mbeanEntry.getAttribute((int)attrIndex);
	    if (Boolean.getBoolean("debug")) 
		System.err.println("GOT: JmxMBeanAttrTable["+
				   mbeanIndex+"."+attrIndex+"]="+
				   entry.getJmxMBeanAttrName());
	} catch (SnmpStatusException x) {
	    if (Boolean.getBoolean("info"))
		System.err.println("Failed to get MBean Attribute entry: " + 
				   rowOid);
	    if (Boolean.getBoolean("debug")) x.printStackTrace();
	    throw x;
	} catch (Exception x) {
	    if (Boolean.getBoolean("info"))
		System.err.println("Failed to add MBean entry: " + rowOid);
	    if (Boolean.getBoolean("debug")) x.printStackTrace();	    
	    final SnmpStatusException sn = 
		new SnmpStatusException(SnmpStatusException.noSuchInstance);
	    sn.initCause(x);
	    throw sn;
	}

	return entry;
    }


}
