/*
 * @(#)file      SnmpMibAgentImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.35
 * @(#)lastedit  04/04/14
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// java import
//
import java.io.Serializable;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.NoSuchElementException;
import java.net.UnknownHostException;

// RI import
//
import javax.management.ObjectName;
import javax.management.MBeanServer;
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.manager.SnmpPeer;
import com.sun.management.snmp.manager.SnmpParameters;
import com.sun.management.snmp.manager.SnmpRequest;
import com.sun.management.snmp.manager.SnmpSession;

// jdmk import
//
import com.sun.management.snmp.agent.SnmpMibAgent;
import com.sun.management.snmp.agent.SnmpMibRequest;


/**
 * The SnmpMibAgentImpl class provides an example of implementation of 
 * an SNMP proxy.
 * It enables an agent to serve remote MIBs 
 * (that is MIBs running outside the VM where the agent is running).
 * In fact, it behaves like a manager on the agent side.
 * 
 * You initialize the SnmpMibAgentImpl invoking the initializeProxy method
 * with the following parameters:
 *      - host: hostname of the remote SNMP subagent you want to query. 
 *      - port: port number to use.
 *      - strOid: root object identifier of the MIB to query (optional).
 *      - name: MIB name (optional).
 */

public class SnmpMibAgentImpl extends SnmpMibAgent implements Serializable {
  
    // INITIALIZATION
    //===============
    /**
     * Initialization of the SnmpMibAgentImpl with no registration in Java DMK.
     * @exception IllegalAccessException The SnmpMibAgentImpl can not be 
     *            initialized.
     */
    public void init() throws IllegalAccessException {
    }
  
    /**
     * Initialization of the SnmpMibAgentImpl with registration in Java DMK.
     * @param server The reference to the MBean server.
     * @param name The object name of this SnmpMibAgentImpl.
     */
    public ObjectName preRegister(MBeanServer server, ObjectName name)
        throws Exception {
          
        // Initialize MBeanServer information.
        //
        this.server = server;
        return name;
    }
  
    /**
     * Initialization of the proxy stuff.
     * @param h The hostname of the remote SNMP subagent.
     * @param p The port number to use.
     * @param strOid The root object identifier of the MIB.
     * @param name The name of the MIB.
     * @exception UnknownHostException The hostname can not be resolved.
     * @exception SnmpStatusException An error occurred while accessing
     *            a MIB node.
     */
    public void initializeProxy(String h, int p, String strOid, String name) 
        throws UnknownHostException, SnmpStatusException {
        
        host = h;
        port = p;
        oid = strOid;
        mibName = name;
        
        // Initialization for SNMP v1 protocol.
        //
        SnmpParameters paramsV1 = new SnmpParameters("public", "private");
        paramsV1.setProtocolVersion(SnmpDefinitions.snmpVersionOne);
        SnmpPeer peerV1 = new SnmpPeer(host, port);
        peerV1.setParams(paramsV1);
        sessionV1 = new SnmpSession("SnmpMibAgentImpl session V1");
        sessionV1.setDefaultPeer(peerV1);
        // Using SNMP v1 protocol, the error is not fixed but is forwarded 
        // to the manager.
        //
        sessionV1.snmpOptions.setPduFixedOnError(false);

        // Initialization for SNMP v2 protocol.
        //
        SnmpParameters paramsV2 = new SnmpParameters("public", "private");
        paramsV2.setProtocolVersion(SnmpDefinitions.snmpVersionTwo);
        SnmpPeer peerV2 = new SnmpPeer(host, port);
        peerV2.setParams(paramsV2);
        // If we meet a problem, we don't retry the request using SNMP v2 
        // but we try the request using SNMP v1.
        //
        peerV2.setMaxTries(0);
        sessionV2 = new SnmpSession("SnmpMibAgentImpl session V2");
        sessionV2.setDefaultPeer(peerV2);
        // Using SNMP v2 protocol, the error is fixed.
        //
        sessionV2.snmpOptions.setPduFixedOnError(true);

        // Initialization for SNMP v2 protocol simulated using SNMP v1 
        // protocol.
        //
        sessionV2WithV1 =
            new SnmpSession("SnmpMibAgentImpl session V2 with V1");
        sessionV2WithV1.setDefaultPeer(peerV1);
        // Simulating SNMP v2 with SNMP v1 protocol, the error is fixed.
        //
        sessionV2WithV1.snmpOptions.setPduFixedOnError(true);
    }
  
    /**
     * Initialization of the proxy stuff.
     * @param h The hostname of the remote SNMP subagent.
     * @param p The port number to use.
     * @param strOid The root object identifier of the MIB.
     * @exception UnknownHostException The hostname can not be resolved.
     * @exception SnmpStatusException An error occurred while accessing 
     *            a MIB node.
     */
    public void initializeProxy(String h, int p, String strOid)
        throws UnknownHostException, SnmpStatusException {
        initializeProxy(h, p, strOid, "SnmpMibAgentImpl");
    }
  
    /**
     * Initialization of the proxy stuff.
     * @param h The hostname of the remote SNMP subagent.
     * @param p The port number to use.
     * @exception UnknownHostException The hostname can not be resolved.
     * @exception SnmpStatusException An error occurred while accessing 
     *            a MIB node.
     */
    public void initializeProxy(String h, int p) 
        throws UnknownHostException, SnmpStatusException {
        initializeProxy(h, p, null, "SnmpMibAgentImpl");
    }
  

    // SNMP PROXY STUFF
    //=================
    /**
     * Implement the get method from the abstract SnmpMibAgent class.
     * If the protocol version is V2 and if the operation fails using 
     * version 2 of SNMP, we try it again using version 1.
     *
     * @param inRequest The SnmpMibRequest object holding the list of variable
     *                  to be retrieved. This list is composed of 
     *                  <CODE>SnmpVarBind</CODE> objects.
     *
     * @return The varbinds in the SnmpMibRequest object are updated with
     *         the proxied agent returned values.
     * @exception SnmpStatusException  An error occurred during the operation.
     */
    public void get(SnmpMibRequest inRequest) throws SnmpStatusException {
                                          
        java.lang.System.out.println("\n>> ================================" +
                                     "=====================================");
        java.lang.System.out.println(">> SnmpMibAgentImpl: Sending get " +
                                     "request to SNMP subagent on " + 
                                     host + " using port " + port);

        // Get the protocol version
        //
        final int version = inRequest.getVersion();

        // Request using SNMP v1 protocol.
        //
        if (version == SnmpDefinitions.snmpVersionOne) {
            get(inRequest, version, sessionV1);
        }
        // Request using SNMP v2 protocol.
        //
        if (version == SnmpDefinitions.snmpVersionTwo) {
            get(inRequest, version, sessionV2);
        }
    }

    /**
     * Implement the set method from the abstract SnmpMibAgent class.
     * If the protocol version is V2 and if the operation fails using 
     * version 2 of SNMP, we try it again using version 1.
     *
     * @param inRequest The SnmpMibRequest object holding the list of 
     *            variables to be retrieved. This list is composed of 
     *            <CODE>SnmpVarBind</CODE> objects.
     *
     * @return The varbinds in the SnmpMibRequest object are updated with
     *         the proxied agent returned values.
     *
     * @exception SnmpStatusException  An error occurred during the operation.
     */
    public void set(SnmpMibRequest inRequest) throws SnmpStatusException {
        
        java.lang.System.out.println("\n>> ================================" +
                                     "=====================================");
        java.lang.System.out.println(">> SnmpMibAgentImpl: Sending set " +
                                     "request to SNMP subagent on " + 
                                     host + " using port " + port);

        // Get the protocol version
        //
        final int version = inRequest.getVersion();

        // Request using SNMP v1 protocol.
        //
        if (version == SnmpDefinitions.snmpVersionOne) {
            set(inRequest, version, sessionV1);
        }
        // Request using SNMP v2 protocol.
        //
        if (version == SnmpDefinitions.snmpVersionTwo) {
            set(inRequest, version, sessionV2);
        }
    }
  
    /**
     * Implement the getNext method from the abstract SnmpMibAgent class.
     * If the protocol version is V2 and if the operation fails using 
     * version 2 of SNMP, we try it again using version 1.
     *
     * @param inRequest The SnmpMibRequest object holding the list of 
     *            OIDs from which the next variables should be retrieved. 
     *            This list is composed of <CODE>SnmpVarBind</CODE> objects.
     *
     * @return The varbinds in the SnmpMibRequest object are updated with
     *         the proxied agent returned values.
     *
     * @exception SnmpStatusException  An error occurred during the operation.
     */
    public void getNext(SnmpMibRequest inRequest) throws SnmpStatusException {
        
        java.lang.System.out.println("\n>> ================================" +
                                     "=====================================");
        java.lang.System.out.println(">> SnmpMibAgentImpl: Sending getNext " +
                                     "request to SNMP subagent on " + 
                                     host + " using port " + port);

        // Get the protocol version
        //
        final int version = inRequest.getVersion();

        // Request using SNMP v1 protocol.
        //
        if (version == SnmpDefinitions.snmpVersionOne) {
            getNext(inRequest, version, sessionV1);
        }
        // Request using SNMP v2 protocol.
        //
        if (version == SnmpDefinitions.snmpVersionTwo) {
            getNext(inRequest, version, sessionV2);
        }
    }
    
    /**
     * Not implemented.
     */
    public void getBulk(SnmpMibRequest inRequest, int nonRepeat, int maxRepeat)
        throws SnmpStatusException {
        return;
    }
  
    /**
     * This method allows to specify both the protocol version and the 
     * session to use for the get request.
     * 
     * If the request was originally sent using SNMP v1, 
     * the session does not fix the error but forwards it to the manager.
     * If the request was sent using SNMP v1 to simulate SNMP v2, 
     * the session fixes the error.
     */
    private void get(SnmpMibRequest inRequest, int version,
                     SnmpSession session) 
        throws SnmpStatusException {
                                          
        // Construction of the SnmpVarBindList.
        //
        final SnmpVarBindList varbindlist =
            new SnmpVarBindList("SnmpMibAgentImpl varbind list",
                                inRequest.getSubList());

        // Send a GET request to the proxied agent and obtain a request handle.
        //
        SnmpRequest request = null;
        try {
            request = session.snmpGetRequest(null, varbindlist);
        } 
        catch (SnmpStatusException e) {
            throw new SnmpStatusException(SnmpDefinitions.snmpRspGenErr, 0);
        }
        java.lang.System.out.println("\nRequest:\n" + request.toString());
            
        // Wait for request completion
        //
        boolean completed = request.waitForCompletion(10000);
        if (completed == false) {
            // If the completion failed using SNMP v1, we give up.
            //
            if (version == SnmpDefinitions.snmpVersionOne) {
                java.lang.System.out.println("\nRequest timed out: check " +
                                             "reachability of subagent.");
                return;
            }
            // If the completion failed using SNMP v2, we try the request 
            // again using SNMP v1.
            //
            if (version == SnmpDefinitions.snmpVersionTwo) {
                java.lang.System.out.println("\n>> SnmpMibAgentImpl: Try to " +
                                             "submit the get request using " +
                                             "SNMP version 1...");
                get(inRequest, SnmpDefinitions.snmpVersionOne, sessionV2WithV1);
                return;
            }
        }

        // Check request result
        //
        final int errorStatus = request.getErrorStatus();
        final int errorIndex = request.getErrorIndex() + 1;
        if (errorStatus != SnmpDefinitions.snmpRspNoError) {
            java.lang.System.out.println("\nError status = " +
                                 SnmpRequest.snmpErrorToString(errorStatus));
            java.lang.System.out.println("Error index = " + errorIndex);
            // If there is an error status using SNMP v1, we throw an 
            // exception.
            //
            if (version == SnmpDefinitions.snmpVersionOne) {
                throw new SnmpStatusException(errorStatus, errorIndex);
            }
            // If there is an error status using SNMP v2, we try the 
            // request again using SNMP v1.
            //
            if (version == SnmpDefinitions.snmpVersionTwo) {
                java.lang.System.out.println("\n>> SnmpMibAgentImpl: Try to " +
                                             "submit the get request using " +
                                             "SNMP version 1...");
                get(inRequest, SnmpDefinitions.snmpVersionOne, sessionV2WithV1);
                return;
            }
        }

        // Display the returned values
        //
        final SnmpVarBindList result = request.getResponseVarBindList();
        java.lang.System.out.println("\nResult: \n" +
                                     result.varBindListToString());

        // Update the inRequest parameter.
        // The varbinds in the result list are expected to be in the same
        // order than in the request, so we can safely loop sequentially
        // over both lists.
        //
        Enumeration l = inRequest.getElements();
        for (Enumeration e = result.elements(); e.hasMoreElements();) {
            SnmpVarBind varres = (SnmpVarBind) e.nextElement();
            SnmpVarBind varbind = (SnmpVarBind) l.nextElement();
            varbind.setSnmpValue(varres.getSnmpValue());
        }
    }

    /**
     * This method allows to specify both the protocol version and the 
     * session to use for the set request.
     * 
     * If the request was originally sent using SNMP v1, 
     * the session does not fix the error but forwards it to the manager.
     * If the request was sent using SNMP v1 to simulate SNMP v2, the 
     * session fixes the error.
     */
    private void set(SnmpMibRequest inRequest, int version,
                     SnmpSession session) 
        throws SnmpStatusException {
        
        // Construction of the SnmpVarBindList.
        //
        final SnmpVarBindList varbindlist =
            new SnmpVarBindList("SnmpMibAgentImpl varbind list",
                                inRequest.getSubList());

        // Send a SET request to the proxied agent and obtain a request 
        // handle.
        //
        SnmpRequest request = null;
        try {
            request = session.snmpSetRequest(null, varbindlist);
        } 
        catch (SnmpStatusException e) {
            throw new SnmpStatusException(SnmpDefinitions.snmpRspGenErr, 0);
        }
        java.lang.System.out.println("\nRequest:\n" + request.toString());
         
        // Wait for request completion
        //
        boolean completed = request.waitForCompletion(10000);
        if (completed == false) {
            // If the completion failed using SNMP v1, we give up.
            //
            if (version == SnmpDefinitions.snmpVersionOne) {
                java.lang.System.out.println("\nRequest timed out: check " +
                                             "reachability of subagent.");
                return;
            }
            // If the completion failed using SNMP v2, we try the request 
            // again using SNMP v1.
            //
            if (version == SnmpDefinitions.snmpVersionTwo) {
                java.lang.System.out.println("\n>> SnmpMibAgentImpl: Try to " +
                                             "submit the set request using " +
                                             "SNMP version 1...");
                set(inRequest, SnmpDefinitions.snmpVersionOne, sessionV2WithV1);
                return;
            }
        }

        // Check the request result
        //
        int errorStatus = request.getErrorStatus();
        int errorIndex = request.getErrorIndex() + 1;
        if (errorStatus != SnmpDefinitions.snmpRspNoError) {
            java.lang.System.out.println("\nError status = " +
                                 SnmpRequest.snmpErrorToString(errorStatus));
            java.lang.System.out.println("Error index = " + errorIndex);
            // If there is an error status using SNMP v1, we throw an 
            // exception.
            //
            if (version == SnmpDefinitions.snmpVersionOne) {
                throw new SnmpStatusException(errorStatus, errorIndex);
            }
            // If there is an error status using SNMP v2, we try the 
            // request again using SNMP v1.
            //
            if (version == SnmpDefinitions.snmpVersionTwo) {
                java.lang.System.out.println("\n>> SnmpMibAgentImpl: Try to " +
                                             "submit the set request using " +
                                             "SNMP version 1...");
                set(inRequest, SnmpDefinitions.snmpVersionOne, sessionV2WithV1);
                return;
            }
        }

        // Display the returned values
        // 
        final SnmpVarBindList result = request.getResponseVarBindList();
        java.lang.System.out.println("\nResult: \n" +
                                     result.varBindListToString());

        // Update the inRequest parameter.
        // The varbinds in the result list are expected to be in the same
        // order than in the request, so we can safely loop sequentially
        // over both lists.
        //
        final Enumeration l = inRequest.getElements();
        for (Enumeration e = result.elements(); e.hasMoreElements();) {
            SnmpVarBind varres = (SnmpVarBind) e.nextElement();
            SnmpVarBind varbind = (SnmpVarBind) l.nextElement();
            varbind.setSnmpValue(varres.getSnmpValue());
        }
    }

    /**
     * This method allows to specify both the protocol version and the 
     * session to use for the getNext request.
     * 
     * If the request was originally sent using SNMP v1, the session 
     * does not fix the error but forwards it to the manager.
     * If the request was sent using SNMP v1 to simulate SNMP v2, the 
     * session fixes the error.
     */
    private void getNext(SnmpMibRequest inRequest, int version,
                         SnmpSession session) 
        throws SnmpStatusException {
        
        // Construction of the SnmpVarBindList.
        //
        final SnmpVarBindList varbindlist =
            new SnmpVarBindList("SnmpMibAgentImpl varbind list",
                                inRequest.getSubList());

        // Send a GET-NEXT request to the proxied agent and obtain a 
        // request handle.
        //
        SnmpRequest request = null;
        try {
            request = session.snmpGetNextRequest(null, varbindlist);
        } 
        catch (SnmpStatusException e) {
            throw new SnmpStatusException(SnmpDefinitions.snmpRspGenErr, 0);
        }
        java.lang.System.out.println("\nRequest:\n" + request.toString());

        // Wait for request completion
        //
        boolean completed = request.waitForCompletion(10000);
        if (completed == false) {
            // If the completion failed using SNMP v1, we give up.
            //
            if (version == SnmpDefinitions.snmpVersionOne) {
                java.lang.System.out.println("\nRequest timed out: check " +
                                             "reachability of subagent.");
                return;
            }
            // If the completion failed using SNMP v2, we try the request
            // again using SNMP v1.
            //
            if (version == SnmpDefinitions.snmpVersionTwo) {
                java.lang.System.out.println("\n>> SnmpMibAgentImpl: Try to " +
                                             "submit the getNext request " +
                                             "using SNMP version 1...");
                getNext(inRequest, SnmpDefinitions.snmpVersionOne,
                        sessionV2WithV1);
                return;
            }
        }

        // Check the request result
        //
        int errorStatus = request.getErrorStatus();
        int errorIndex = request.getErrorIndex() + 1;
        if (errorStatus != SnmpDefinitions.snmpRspNoError) {
            java.lang.System.out.println("\nError status = " +
                                 SnmpRequest.snmpErrorToString(errorStatus));
            java.lang.System.out.println("Error index = " + errorIndex);
            // If there is an error status using SNMP v1, we throw an 
            // exception.
            //
            if (version == SnmpDefinitions.snmpVersionOne) {
                throw new SnmpStatusException(errorStatus, errorIndex);
            }
            // If there is an error status using SNMP v2, we try the 
            // request again using SNMP v1.
            //
            if (version == SnmpDefinitions.snmpVersionTwo) {
                java.lang.System.out.println("\n>> SnmpMibAgentImpl: Try to " +
                                             "submit the getNext request " +
                                             "using SNMP version 1...");
                getNext(inRequest, SnmpDefinitions.snmpVersionOne,
                        sessionV2WithV1);
                return;
            }
        }

        // Display the returned values
        //
        final SnmpVarBindList result = request.getResponseVarBindList();
        java.lang.System.out.println("\nResult: \n" +
                                     result.varBindListToString());

        // Update the inRequest parameter.
        // The varbinds in the result list are expected to be in the same
        // order than in the request, so we can safely loop sequentially
        // over both lists. Since we parse the result of a GET-NEXT
        // request, we must also copy the value of the returned OIDs
        //
        Enumeration l = inRequest.getElements();
        for (Enumeration e = result.elements(); e.hasMoreElements();) {
            SnmpVarBind varres = (SnmpVarBind) e.nextElement();
            SnmpVarBind varbind = (SnmpVarBind) l.nextElement();
            varbind.setOid(varres.getOid());
            varbind.setSnmpValue(varres.getSnmpValue());
        }
    }
  
    /**
     * Parse the string oid and return the corresponding root object 
     * identifier.
     */
    private long[] resolveOidString(String s) {
        SnmpOid oid = new SnmpOid(s);
        return oid.longValue();
    }
  
    // GETTER/SETTER
    //==============
    
    /**
     * Return the root object identifier of the MIB.
     */
    public long[] getRootOid() {
        
        // A string oid has been initialized by the user.
        //
        if (oid != null) {
            rootOid = resolveOidString(oid);
        }
        
        // No initialization of the oid string has been done by the user.
        //
        else {
            try {
                final Vector varBindList = new Vector();
                final SnmpOid varBindOid = new SnmpOid("0.0");
                final SnmpVarBind varBind = new SnmpVarBind(varBindOid, null);
                final int version = SnmpDefinitions.snmpVersionTwo;
                varBindList.addElement(varBind);

                SnmpMibRequest inRequest =
                SnmpMibAgent.newMibRequest(null, varBindList, version, null);
                
                getNext(inRequest);
                
                // As the 3 last ids cannot be part of the root oid 
                // (either if the string represent a "simple" variable or 
                // a table instance), we remove them from the string oid.
                // Note that this will not always work, but we're trying
                // our best guess. To avoid to enter this code, it is 
                // recommended to give the string OID at proxy initialization.
                //
                String strOid = varBind.getOid().toString();
                int index = strOid.lastIndexOf('.');
                for (int i = 1; i <= 3; i++) {
                    strOid = strOid.substring(0, index);
                    index = strOid.lastIndexOf('.');
                }
                rootOid = resolveOidString(strOid);
            }
            catch (Exception e) {
                java.lang.System.out.println("\nProblem when resolving the " +
                                             "string oid...");
            }
        }
        
        return rootOid;
    }
  
    public String getHost() {
        return host;
    }
    public void setHost(String s) {
        host = s;
    }
  
    public int getPort() {
        return port;
    }
    public void setPort(int i) {
        port = i;
    }
  
    public String getOid() {
        return oid;
    }
    public void setOid(String s) {
        oid = s;
    }
  
    // PRIVATE VARIABLES
    //==================
    private String host = null;
    private int port = 0;
    private String oid = null;
  
    private SnmpSession sessionV1 = null;
    private SnmpSession sessionV2 = null;
    private SnmpSession sessionV2WithV1 = null;
  
    private transient long[] rootOid = null;
}
