/*
 * @(#)file      KernelStat.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.5
 * @(#)lastedit  04/02/16
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package com.sun.jdmk.contrib.kstat;

/**
 * JNI interface for kernel statistics
 * based on the Solaris kstat(3M) interface.
 * The native library implementing the native method is <b>libstat.so</b>
 * provided under<br>
 * <CODE><i>exampleDir</i>/Kstat/lib/libstat.so</CODE>
 *
 */

public class KernelStat {

    static {
        System.loadLibrary("stat");
        init();
    }

    /**
     * all instances: any module instance number. 
     */
    public static final int ANY_INSTANCE = -1;

    /**
     * no kstat data type: no data type specified or available.
     */
    public static final int STAT_NONE   = -1;
    /**
     * kstat raw data type.
     */
    public static final int STAT_RAW   = 0;
    /**
     * kstat named data type.
     */
    public static final int STAT_NAMED = 1;
    /**
     * kstat intr data type.
     */
    public static final int STAT_INTR  = 2;
    /**
     * kstat IO data type.
     */
    public static final int STAT_IO    = 3;
    /**
     * kstat timer data type.
     */
    public static final int STAT_TIMER = 4;

    /**
     * Kstat separator.
     **/
    public static String SEPARATOR = ";";

    String moduleName = null;

    // Constructor
    //
    public KernelStat(String moduleName){
        this.moduleName = moduleName;
    }

    /**
     * Return all the (moduleName+instance name+instance) kstat contained 
     * in the current kernel.
     */
    public String getHeaders() {
        return statGetList(moduleName, ANY_INSTANCE , null, STAT_NONE);
    }

    /**
     * Return all the (moduleName+instance name+instance) kstat contained 
     * in the current kernel.
     */
    public String getHeaders(String instanceName) {
        return statGetList(moduleName, ANY_INSTANCE , instanceName, STAT_NONE);
    }

    /**
     * Return all the (moduleName+instance name+instance) kstat contained 
     * in the current kernel.
     */
    public String getHeaders(int instance, String instanceName) {
        return statGetList(moduleName, instance, instanceName, STAT_NONE);
    }

    /**
     * Return all the (moduleName+instance name+instance) kstat contained 
     * in the current kernel.
     */
    public String getHeaders(int instance, String instanceName, int type) {
        return statGetList(moduleName, instance, instanceName, type);
    }

    /**
     * Returns the list of values for the passed property. All kstat 
     * statistics are parsed.
     */
    public String statGetValueList(String propertyName) {
        return statValueList(moduleName, propertyName);
    }
    
    /**
     * Returns the list of values for all the kstat of the current module.
     */
    public String statGetValuesList() {
        return statValuesList(moduleName);
    }
    
    /**
     * Returns the list of values for all the kstat of the current module.
     */
    public String statGetValues(int instance, 
                                String name) {
        return statValues(moduleName, instance, name);
    }

    public byte[] statGetRawDataValue(int instance,
                                      String instanceName,
                                      String propName) 
    {
        throw new IllegalArgumentException("kstat data type not supported");
    }

    public int[] statGetIntrDataValue(int instance,
                                      String instanceName,
                                      String propName) 
    {
        throw new IllegalArgumentException("kstat data type not supported");
    }

    public byte[] statGetTimerDataValue(int instance,
                                        String instanceName,
                                        String propName) 
    {
        throw new IllegalArgumentException("kstat data type not supported");
    }

    /**
     * Idem native method, handle exceptions and converts to Integer.
     */
    public int statGetIntegerValue(int instance,
                                   String instanceName,
                                   String propName) {
        String strValue= null;
        try {
            if (isDebugOn()) {
                final String i = (instance<0?":":"("+instance+"):");
                debug("statGetIntegerValue",moduleName+i+
                      instanceName+"->"+propName);
            }
            strValue = statGetValue(moduleName,
                                    instance,
                                    instanceName,
                                    propName);
        } catch(Exception e) {
            debug("statGetIntegerValue",e);
        }
        if (strValue != null) {
            try {
                return Integer.parseInt(strValue);
            } catch (NumberFormatException e) {
                debug("statGetIntegerValue","bad format: " + strValue);
                debug("statGetIntegerValue",e);
                return -1;
            }
        }
        return -1;
    }
    
    /**
     * Idem native method, handle exceptions and converts to int.
     * Note that in Java, integer are signed. 
     * Unsigned integer values that exceed Integer.MAX_VALUE will thus be
     * represented by negative values.
     */
    public int statGetUnsignedIntegerValue(int instance,
                                            String instanceName,
                                            String propName) {
        String strValue= null;
        try {
            if (isDebugOn()) {
                final String i = (instance<0?":":"("+instance+"):");
                debug("statGetUnsignedIntegerValue",moduleName+i+
                      instanceName+"->"+propName);
            }
            strValue = statGetValue(moduleName,
                                    instance,
                                    instanceName,
                                    propName);
        } catch(Exception e) {
            debug("statGetUnsignedIntegerValue",e);
        }

        if (strValue != null) {
            try {
               final int result = Integer.parseInt(strValue);
               // Here we should do something to handle the sign...
               if (result < 0) {
                   System.out.println("KernelStat::" +
                                      "statGetUnsignedIntegerValue::" +
                                      "Warning uint overflow: " +  result);
               }
               return result;
            } catch (NumberFormatException e) {
                debug("statGetUnsignedIntegerValue",
                      "bad format: " + strValue);
                debug("statGetUnsignedIntegerValue",e);
                return -1;
            }
        }
        return -1;
    }
    
    /**
     * Idem native method, handle exceptions and converts to Long.
     */
    public long statGetLongValue(int instance,
                                 String instanceName,
                                 String propName) {
        String strValue= null;
        try {
            if (isDebugOn()) {
                final String i = (instance<0?":":"("+instance+"):");
                debug("statGetLongValue",moduleName+i+
                      instanceName+"->"+propName);
            }
            strValue = statGetValue(moduleName,
                                    instance,
                                    instanceName,
                                    propName);
        } catch(Exception e) {
            debug("statGetLongValue",e);
        }
        if (strValue != null) {
            try {
                return Long.parseLong(strValue);
            } catch (NumberFormatException e) {
                debug("statGetLongValue","bad format: " + strValue);
                debug("statGetLongValue",e);
                return -1;
            }
        }
        return -1;
    }
    
    /**
     * Idem native method, handle exceptions and converts to Long.
     * Note that in Java, long are signed. 
     * Unsigned long values that exceed Long.MAX_VALUE will thus be
     * represented by negative values.
     */
    public long statGetUnsignedLongValue(int instance,
                                         String instanceName,
                                         String propName) {
        String strValue= null;
        try {
            if (isDebugOn()) {
                final String i = (instance<0?":":"("+instance+"):");
                debug("statGetUnsignedLongValue",moduleName+i+
                      instanceName+"->"+propName);
            }
            strValue = statGetValue(moduleName,
                                    instance,
                                    instanceName,
                                    propName);
        } catch(Exception e) {
            debug("statGetUnsignedLongValue",e);
        }

        if (strValue != null) {
            try {
               final long result = Long.parseLong(strValue);
               // Here we should do something to handle the sign...
               if (result < 0) {
                   System.out.println("KernelStat::" +
                                      "statGetUnsignedLongValue::" +
                                      "Warning ulong overflow: " +  result);
               }
               return result;
            } catch (NumberFormatException e) {
                debug("statGetUnsignedLongValue","bad format: " + strValue);
                debug("statGetUnsignedLongValue",e);
                return -1;
            }
        }
        return -1;
    }
    

    /**
     * Idem native method, handle exceptions and converts to an array
     * of 4 int's (128 bits).
     * Note that in Java, integer are signed. 
     * Unsigned integer values that exceed Integer.MAX_VALUE will thus be
     * represented by negative values.
     */
    public int[] statGetDataCharValue(int instance,
                                      String instanceName,
                                      String propName) {
         String strValue= null;
         try {
            if (isDebugOn()) {
                final String i = (instance<0?":":"("+instance+"):");
                debug("statGetDataCharValue",moduleName+i+
                      instanceName+"->"+propName);
            }
            strValue = statGetValue(moduleName,
                                    instance,
                                    instanceName,
                                    propName);
        } catch(Exception e) {
            debug("statGetDataCharValue",e);
        }

        if (strValue != null) {
             if (isDebugOn()) 
             debug("statGetDataCharValue","got \"" + strValue + "\"");
            try {
               final int[] result = new int[4];
               int start=0;
               for (int i=0;i<4;i++) {
                   final int index = 
                       (i<3?strValue.indexOf(".",start):strValue.length());
                   final String str = strValue.substring(start,index);
                   final int res = Integer.parseInt(str);
                   // Here we should do something to handle the sign...
                   if (res < 0) {
                       System.out.println("KernelStat::" +
                                          "statGetDataCharValue::" +
                                          "Warning uint overflow: " + res);
                   }
                   result[i] = res;
                   start=index+1;
               }
               return result;
            } catch (NumberFormatException e) {
                debug("statGetUnsignedLongValue","bad format: " + strValue);
                debug("statGetUnsignedLongValue",e);
                return null;
            }
        }
        return null;
    }
    

    /**  
     * Idem native method, handles exceptions.
     */
    public String statGetStringValue(int instance,
                                     String instanceName,
                                     String propName) {
        String strValue= null;
        try {
            if (isDebugOn()) {
                final String i = (instance<0?":":"("+instance+"):");
                debug("statGetStringValue",moduleName+i+
                      instanceName+"->"+propName);
            }
            strValue = statGetValue(moduleName,
                                    instance,
                                    instanceName,
                                    propName);
        } catch(Exception e) {
            debug("statGetStringValue",e);
        }
        return strValue;
    }
  
    /**
     * Return the kstat data type associated with the named instance.
     * Return STAT_NONE if the kstat data type could not be determined.
     * @param The name of the kstat instance.
     * @return {@link #STAT_NONE} or the retrieved kstat data type.
     **/
    public int statGetType(String instanceName) {
        try {
            if (isDebugOn()) {
                debug("statGetType",moduleName+":"+
                      instanceName);
            }
              
            return statType(moduleName,STAT_NONE,instanceName);
        } catch (Exception e) {
            debug("statGetType",e);
        }
        return STAT_NONE;
    }
    
    /**
     * Return the kstat data type associated with the named instance.
     * Return STAT_NONE if the kstat data type could not be determined.
     * @param The module instance number.
     * @param The name of the kstat instance in that module instance.
     * @return {@link #STAT_NONE} or the retrieved kstat data type.
     **/
    public int statGetType(int instance, String instanceName) {
        try {
            if (isDebugOn()) {
                final String i = (instance<0?":":"("+instance+"):");
                debug("statGetType",moduleName+i+instanceName);
            }
            return statType(moduleName,instance,instanceName);
        } catch (Exception e) {
            debug("statGetType",e);
        }
        return STAT_NONE;
    }

    /**
     * Return the names & types of the instance properties.
     **/
    public String statGetProperties(String instanceName) {
        try {
            if (isDebugOn()) 
                debug("statGetProperties",moduleName+":"+
                      instanceName);
            return statProperties(moduleName,ANY_INSTANCE,instanceName);
        } catch (Exception e) {
            debug("statGetProperties",e);
        }
        return null;
    }

    /**
     * Return the names & types of the instance properties.
     **/
    public String statGetProperties(int instance,
                                    String instanceName) {
        if (isDebugOn()) 
            debug("statGetProperties",moduleName+"("+instance+"):"+
                  instanceName);
        try {
            return statProperties(moduleName,instance,instanceName);
        } catch (Exception e) {
            debug("statGetProperties",e);
        }
        return null;
    }

    private native String statProperties(String module, 
                                         int    instance,
                                         String name);

    /**
     * Returns the kstat data type of the queried module+instance+name 
     * kernel statistics.
     **/
    private native int statType(String module,
                                int    instance,
                                String name);

    /**
     * Returns the list of kstat elements according to the passed parameters.
     */
    private native String statGetList(String module, 
                                      int    instance,
                                      String instName,
                                      int type);

    /**
     * Returns the list of values for the passed property. All kstat are 
     * parsed.
     */
    private native String statValueList(String module,
                                        String propertyName);
    
    /**
     * Returns the list of all values for all the kstat of the provided module.
     */
    private native String statValuesList(String module);
    
    /**
     * Returns the list of all values for the kstat of the provided name.
     */
    private native String statValues(String module, 
                                     int instance, 
                                     String name);
    
    /**
     * Returns the kstat value according to the passed parameters.
     */
    private native String statGetValue(String module, 
                                       int instance,
                                       String name,
                                       String property);

    private static native void init();

    //
    // Debug
    //

    private static final boolean debugOn = false;
    private static final String debugTag = "KernelStat";

    private final static boolean isDebugOn() {
        return debugOn;
    }

    private static final void debug(String func, String info) {
        if (isDebugOn())
            System.out.println(debugTag + "::" + func + "::" + info);
    }

    private static final void debug(String func, Throwable t) {
        if (isDebugOn()) {
            System.out.println(debugTag + "::" + func + "::" + t.getMessage());
            t.printStackTrace();
        }
    }
}
